#####
# Initialize the pySlip package
#####

import os
import sys

__version__ = '3.0'

from pyslip import *


## add built-in tilesets to sys.path
#DefaultTilesets = 'tilesets'
#CurrentPath = os.path.dirname(os.path.abspath(__file__))
#
#sys.path.append(os.path.join(CurrentPath, DefaultTilesets))
